@@ -18,6 +18,10 @@
 
 #include <stdlib.h>
 
+#ifdef REM_USE_WCHAR
+#include <wctype.h>
+#endif
+
 #include "types.h"
 #include "protos.h"
 #include "expr.h"
@@ -30,11 +34,15 @@ typedef struct cal_entry {
     struct cal_entry *next;
     char const *text;
     char const *pos;
+#ifdef REM_USE_WCHAR
+    wchar_t const *wc_text;
+    wchar_t const *wc_pos;
+#endif
     int is_color;
     int r, g, b;
     int time;
     int priority;
-    char tag[TAG_LEN+1];
+    DynamicBuffer tags;
     char passthru[PASSTHRU_LEN+1];
     int duration;
     char const *filename;
@@ -45,17 +53,32 @@ typedef struct cal_entry {
 struct line_drawing {
   char const *graphics_on;
   char const *graphics_off;
-  char tlr, bl, tbl, blr, tblr, tr, tb, br, tbr, tl, lr;
+  char *tlr, *bl, *tbl, *blr, *tblr, *tr, *tb, *br, *tbr, *tl, *lr;
 };
 
 static struct line_drawing NormalDrawing = {
-  "", "", '+', '+', '+', '+', '+', '+', '|', '+', '+', '+', '-'
+  "", "", "+", "+", "+", "+", "+", "+", "|", "+", "+", "+", "-"
 };
 
 static struct line_drawing VT100Drawing = {
   "\x1B(0", "\x1B(B",
-  '\x76', '\x6b', '\x75', '\x77', '\x6e', '\x6d', '\x78',
-  '\x6c', '\x74', '\x6a', '\x71'
+  "\x76", "\x6b", "\x75", "\x77", "\x6e", "\x6d", "\x78",
+  "\x6c", "\x74", "\x6a", "\x71"
+};
+
+static struct line_drawing UTF8Drawing = {
+    "", "",
+    "\xe2\x94\xb4",
+    "\xe2\x94\x90",
+    "\xe2\x94\xa4",
+    "\xe2\x94\xac",
+    "\xe2\x94\xbc",
+    "\xe2\x94\x94",
+    "\xe2\x94\x82",
+    "\xe2\x94\x8c",
+    "\xe2\x94\x9c",
+    "\xe2\x94\x98",
+    "\xe2\x94\x80"
 };
 
 static char *VT100Colors[2][2][2][2] /* [Br][R][G][B] */ = {
@@ -108,7 +131,7 @@ static char *VT100Colors[2][2][2][2] /* [Br][R][G][B] */ = {
 };
 
 static struct line_drawing *linestruct;
-#define DRAW(x) putchar(linestruct->x)
+#define DRAW(x) fputs(linestruct->x, stdout)
 
 /* Global variables */
 static CalEntry *CalColumn[7];
@@ -124,7 +147,7 @@ static int WriteCalendarRow (void);
 static void WriteWeekHeaderLine (void);
 static void WritePostHeaderLine (void);
 static void PrintLeft (char const *s, int width, char pad);
-static void PrintCentered (char const *s, int width, char pad);
+static void PrintCentered (char const *s, int width, char *pad);
 static int WriteOneCalLine (void);
 static int WriteOneColLine (int col);
 static void GenerateCalEntries (int col);
@@ -137,6 +160,40 @@ static void WriteBottomCalLine (void);
 static void WriteIntermediateCalLine (void);
 static void WriteCalDays (void);
 
+#ifdef REM_USE_WCHAR
+static void PutWideChar(wchar_t const wc)
+{
+    char buf[MB_CUR_MAX+1];
+    int len;
+
+    len = wctomb(buf, wc);
+    if (len > 0) {
+	buf[len] = 0;
+	fputs(buf, stdout);
+    }
+}
+#endif
+
+static int make_wchar_versions(CalEntry *e)
+{
+#ifdef REM_USE_WCHAR
+    size_t len;
+    wchar_t *buf;
+    len = mbstowcs(NULL, e->text, 0);
+    if (len == (size_t) -1) return 0;
+
+    buf = calloc(len+1, sizeof(wchar_t));
+    if (!buf) return 0;
+
+    (void) mbstowcs(buf, e->text, len+1);
+    e->wc_text = buf;
+    e->wc_pos = buf;
+    return 1;
+#else
+    return 1;
+#endif
+}
+
 static void gon(void)
 {
   printf("%s", linestruct->graphics_on);
@@ -179,7 +236,9 @@ void ProduceCalendar(void)
 {
     int y, m, d;
 
-    if (UseVTChars) {
+    if (UseUTF8Chars) {
+	linestruct = &UTF8Drawing;
+    } else if (UseVTChars) {
         linestruct = &VT100Drawing;
     } else {
         linestruct = &NormalDrawing;
@@ -460,20 +519,55 @@ static void PrintLeft(char const *s, int width, char pad)
 /*                                                             */
 /*  PrintCentered                                              */
 /*                                                             */
-/*  Center a piec of text                                      */
+/*  Center a piece of text                                      */
 /*                                                             */
 /***************************************************************/
-static void PrintCentered(char const *s, int width, char pad)
+static void PrintCentered(char const *s, int width, char *pad)
 {
+#ifndef REM_USE_WCHAR
     int len = strlen(s);
     int d = (width - len) / 2;
     int i;
 
-    for (i=0; i<d; i++) PutChar(pad);
+    for (i=0; i<d; i++) fputs(pad, stdout);
     for (i=0; i<width; i++) {
 	if (*s) PutChar(*s++); else break;
     }
-    for (i=d+len; i<width; i++) PutChar(pad);
+    for (i=d+len; i<width; i++) fputs(pad, stdout);
+#else
+    size_t len = mbstowcs(NULL, s, 0);
+    int i;
+    wchar_t static_buf[128];
+    wchar_t *buf;
+    wchar_t *ws;
+    int d;
+
+    if (!len) {
+	for (i=0; i<width; i++) {
+	    fputs(pad, stdout);
+	}
+	return;
+    }
+    if (len + 1 <= 128) {
+	buf = static_buf;
+    } else {
+	buf = calloc(len+1, sizeof(wchar_t));
+	if (!buf) {
+	    /* Uh-oh... cannot recover */
+	    fprintf(stderr, "%s\n", ErrMsg[E_NO_MEM]);
+	    exit(1);
+	}
+    }
+    (void) mbstowcs(buf, s, len+1);
+    d = (width - len) / 2;
+    ws = buf;
+    for (i=0; i<d; i++) fputs(pad, stdout);
+    for (i=0; i<width; i++) {
+	if (*ws) PutWideChar(*ws++); else break;
+    }
+    for (i=d+len; i<width; i++) fputs(pad, stdout);
+    if (buf != static_buf) free(buf);
+#endif
 }
 
 /***************************************************************/
@@ -494,7 +588,7 @@ static int WriteOneCalLine(void)
 	if (CalColumn[i]) {
 	    if (WriteOneColLine(i)) done = 0;
 	} else {
-	    PrintCentered("", ColSpaces, ' ');
+	    PrintCentered("", ColSpaces, " ");
 	}
 	gon();
 	DRAW(tb);
@@ -519,74 +613,158 @@ static int WriteOneColLine(int col)
     CalEntry *e = CalColumn[col];
     char const *s;
     char const *space;
+
+#ifdef REM_USE_WCHAR
+    wchar_t const *ws;
+    wchar_t const *wspace;
+#endif
+
     int numwritten = 0;
 
-/* Print as many characters as possible within the column */
-    space = NULL;
-    s = e->pos;
+    /* Print as many characters as possible within the column */
+#ifdef REM_USE_WCHAR
+    if (e->wc_text) {
+	wspace = NULL;
+	ws = e->wc_pos;
 
-/* If we're at the end, and there's another entry, do a blank line and move
-   to next entry. */
-    if (!*s && e->next) {
-	PrintLeft("", ColSpaces, ' ');
-	CalColumn[col] = e->next;
-	free((char *) e->text);
-	free((char *) e->filename);
-	free(e);
-	return 1;
-    }
-
-/* Find the last space char within the column. */
-    while (s - e->pos <= ColSpaces) {
-	if (!*s) {space = s; break;}
-	if (*s == ' ') space = s;
-	s++;
-    }
-
-/* Colorize reminder if necessary */
-    if (UseVTColors && e->is_color) {
-        Colorize(e);
-    }
-
-/* If we couldn't find a space char, print what we have. */
-    if (!space) {
-	for (s = e->pos; s - e->pos < ColSpaces; s++) {
-	    if (!*s) break;
-	    numwritten++;
-	    PutChar(*s);
+	/* If we're at the end, and there's another entry, do a blank
+	   line and move to next entry. */
+	if (!*ws && e->next) {
+	    PrintLeft("", ColSpaces, ' ');
+	    CalColumn[col] = e->next;
+	    free((void *)e->text);
+	    free((void *)e->filename);
+	    if (e->wc_text) free((void *)e->wc_text);
+	    free(e);
+	    return 1;
 	}
-	e->pos = s;
-    } else {
 
-/* We found a space - print everything before it. */
-	for (s = e->pos; s<space; s++) {
-	    if (!*s) break;
-	    numwritten++;
-	    PutChar(*s);
+	/* Find the last space char within the column. */
+	while (ws - e->wc_pos <= ColSpaces) {
+	    if (!*ws) {wspace = ws; break;}
+	    if (iswspace(*ws)) wspace = ws;
+	    ws++;
 	}
-    }
 
-/* Decolorize reminder if necessary */
-    if (UseVTColors && e->is_color) {
-        Decolorize();
-    }
+	/* Colorize reminder if necessary */
+	if (UseVTColors && e->is_color) {
+	    Colorize(e);
+	}
 
-/* Flesh out the rest of the column */
-    while(numwritten++ < ColSpaces) PutChar(' ');
+	/* If we couldn't find a space char, print what we have. */
+	if (!wspace) {
+	    for (ws = e->wc_pos; ws - e->wc_pos < ColSpaces; ws++) {
+		if (!*ws) break;
+		numwritten++;
+		PutWideChar(*ws);
+	    }
+	    e->wc_pos = ws;
+	} else {
+	    /* We found a space - print everything before it. */
+	    for (ws = e->wc_pos; ws<wspace; ws++) {
+		if (!*ws) break;
+		numwritten++;
+		PutWideChar(*ws);
+	    }
+	}
 
-/* Skip any spaces before next word */
-    while (*s == ' ') s++;
+	/* Decolorize reminder if necessary */
+	if (UseVTColors && e->is_color) {
+	    Decolorize();
+	}
 
-/* If done, free memory if no next entry. */
-    if (!*s && !e->next) {
-	CalColumn[col] = e->next;
-	free((char *) e->text);
-	free((char *) e->filename);
-	free(e);
+	/* Flesh out the rest of the column */
+	while(numwritten++ < ColSpaces) PutChar(' ');
+
+	/* Skip any spaces before next word */
+	while (iswspace(*ws)) ws++;
+
+	/* If done, free memory if no next entry. */
+	if (!*ws && !e->next) {
+	    CalColumn[col] = e->next;
+	    free((void *)e->text);
+	    free((void *)e->filename);
+	    if (e->wc_text) free((void *)e->wc_text);
+	    free(e);
+	} else {
+	    e->wc_pos = ws;
+	}
+	if (CalColumn[col]) return 1; else return 0;
     } else {
-	e->pos = s;
+#endif
+	space = NULL;
+	s = e->pos;
+
+	/* If we're at the end, and there's another entry, do a blank
+	   line and move to next entry. */
+	if (!*s && e->next) {
+	    PrintLeft("", ColSpaces, ' ');
+	    CalColumn[col] = e->next;
+	    free((void *)e->text);
+	    free((void *)e->filename);
+#ifdef REM_USE_WCHAR
+	    if (e->wc_text) free((void *)e->wc_text);
+#endif
+	    free(e);
+	    return 1;
+	}
+
+	/* Find the last space char within the column. */
+	while (s - e->pos <= ColSpaces) {
+	    if (!*s) {space = s; break;}
+	    if (*s == ' ') space = s;
+	    s++;
+	}
+
+	/* Colorize reminder if necessary */
+	if (UseVTColors && e->is_color) {
+	    Colorize(e);
+	}
+
+	/* If we couldn't find a space char, print what we have. */
+	if (!space) {
+	    for (s = e->pos; s - e->pos < ColSpaces; s++) {
+		if (!*s) break;
+		numwritten++;
+		PutChar(*s);
+	    }
+	    e->pos = s;
+	} else {
+	    /* We found a space - print everything before it. */
+	    for (s = e->pos; s<space; s++) {
+		if (!*s) break;
+		numwritten++;
+		PutChar(*s);
+	    }
+	}
+
+	/* Decolorize reminder if necessary */
+	if (UseVTColors && e->is_color) {
+	    Decolorize();
+	}
+
+	/* Flesh out the rest of the column */
+	while(numwritten++ < ColSpaces) PutChar(' ');
+
+	/* Skip any spaces before next word */
+	while (*s == ' ') s++;
+
+	/* If done, free memory if no next entry. */
+	if (!*s && !e->next) {
+	    CalColumn[col] = e->next;
+	    free((void *)e->text);
+	    free((void *)e->filename);
+#ifdef REM_USE_WCHAR
+	    if (e->wc_text) free((void *)e->wc_text);
+#endif
+	    free(e);
+	} else {
+	    e->pos = s;
+	}
+	if (CalColumn[col]) return 1; else return 0;
+#ifdef REM_USE_WCHAR
     }
-    if (CalColumn[col]) return 1; else return 0;
+#endif
 }
 
 /***************************************************************/
@@ -714,7 +892,7 @@ static void WriteCalHeader(void)
     gon();
     DRAW(tb);
     goff();
-    PrintCentered(buf, CalWidth-2, ' ');
+    PrintCentered(buf, CalWidth-2, " ");
     gon();
     DRAW(tb);
     goff();
@@ -763,29 +941,52 @@ static int DoCalRem(ParsePtr p, int col)
     DBufInit(&pre_buf);
 
     /* Parse the trigger date and time */
-    if ( (r=ParseRem(p, &trig, &tim, 1)) ) return r;
+    if ( (r=ParseRem(p, &trig, &tim, 1)) ) {
+	FreeTrig(&trig);
+	return r;
+    }
 
 /* Don't include timed reminders in calendar if -a option supplied. */
-    if (DontIssueAts && tim.ttime != NO_TIME) return OK;
-    if (trig.typ == NO_TYPE) return E_EOLN;
+    if (DontIssueAts && tim.ttime != NO_TIME) {
+	FreeTrig(&trig);
+	return OK;
+    }
+    if (trig.typ == NO_TYPE) {
+	FreeTrig(&trig);
+	return E_EOLN;
+    }
     if (trig.typ == SAT_TYPE) {
 	r=DoSatRemind(&trig, &tim, p);
 	if (r) {
+	    FreeTrig(&trig);
 	    if (r == E_EXPIRED) return OK;
 	    return r;
 	}
-	if (!LastTrigValid) return OK;
+	if (!LastTrigValid) {
+	    FreeTrig(&trig);
+	    return OK;
+	}
 	r=ParseToken(p, &buf);
-	if (r) return r;
+	if (r) {
+	    FreeTrig(&trig);
+	    return r;
+	}
 	FindToken(DBufValue(&buf), &tok);
 	DBufFree(&buf);
-	if (tok.type == T_Empty || tok.type == T_Comment) return OK;
-	if (tok.type != T_RemType || tok.val == SAT_TYPE) return E_PARSE_ERR;
+	if (tok.type == T_Empty || tok.type == T_Comment) {
+	    FreeTrig(&trig);
+	    return OK;
+	}
+	if (tok.type != T_RemType || tok.val == SAT_TYPE) {
+	    FreeTrig(&trig);
+	    return E_PARSE_ERR;
+	}
 	if (tok.val == PASSTHRU_TYPE) {
 	    r=ParseToken(p, &buf);
 	    if (r) return r;
 	    if (!DBufLen(&buf)) {
 		DBufFree(&buf);
+		FreeTrig(&trig);
 		return E_EOLN;
 	    }
 	    StrnCpy(trig.passthru, DBufValue(&buf), PASSTHRU_LEN);
@@ -793,11 +994,17 @@ static int DoCalRem(ParsePtr p, int col)
 	}
 	trig.typ = tok.val;
 	jul = LastTriggerDate;
-	if (!LastTrigValid) return OK;
+	if (!LastTrigValid) {
+	    FreeTrig(&trig);
+	    return OK;
+	}
     } else {
 	/* Calculate the trigger date */
 	jul = ComputeTrigger(trig.scanfrom, &trig, &r, 1);
-	if (r) return r;
+	if (r) {
+	    FreeTrig(&trig);
+	    return r;
+	}
     }
 
     /* Convert PS and PSF to PASSTHRU */
@@ -809,39 +1016,50 @@ static int DoCalRem(ParsePtr p, int col)
 	trig.typ = PASSTHRU_TYPE;
     }
     if (trig.typ == PASSTHRU_TYPE) {
-      if (!PsCal && strcmp(trig.passthru, "COLOR")) return OK;
-      if (!strcmp(trig.passthru, "COLOR")) {
-  	  is_color = 1;
-	  /* Strip off the three color numbers */
-	  DBufFree(&buf);
-	  r=ParseToken(p, &buf);
-	  DBufPuts(&pre_buf, DBufValue(&buf));
-	  DBufPutc(&pre_buf, ' ');
-	  DBufFree(&buf);
-	  if (r) return r;
-	  r=ParseToken(p, &buf);
-	  DBufPuts(&pre_buf, DBufValue(&buf));
-	  DBufPutc(&pre_buf, ' ');
-	  DBufFree(&buf);
-	  if (r) return r;
-	  r=ParseToken(p, &buf);
-	  DBufPuts(&pre_buf, DBufValue(&buf));
-	  DBufPutc(&pre_buf, ' ');
-	  DBufFree(&buf);
-	  if (r) return r;
-	  (void) sscanf(DBufValue(&pre_buf), "%d %d %d",
-			&col_r, &col_g, &col_b);
-	  if (col_r < 0) col_r = 0;
-	  else if (col_r > 255) col_r = 255;
-	  if (col_g < 0) col_g = 0;
-	  else if (col_g > 255) col_g = 255;
-	  if (col_b < 0) col_b = 0;
-	  else if (col_b > 255) col_b = 255;
-
-	  if (!PsCal && !DoSimpleCalendar) {
-	      DBufFree(&pre_buf);
-	  }
-      }
+	if (!PsCal && strcmp(trig.passthru, "COLOR")) {
+	    FreeTrig(&trig);
+	    return OK;
+	}
+	if (!strcmp(trig.passthru, "COLOR")) {
+	    is_color = 1;
+	    /* Strip off the three color numbers */
+	    DBufFree(&buf);
+	    r=ParseToken(p, &buf);
+	    DBufPuts(&pre_buf, DBufValue(&buf));
+	    DBufPutc(&pre_buf, ' ');
+	    DBufFree(&buf);
+	    if (r) {
+		FreeTrig(&trig);
+		return r;
+	    }
+	    r=ParseToken(p, &buf);
+	    DBufPuts(&pre_buf, DBufValue(&buf));
+	    DBufPutc(&pre_buf, ' ');
+	    DBufFree(&buf);
+	    if (r) {
+		FreeTrig(&trig);
+		return r;
+	    }
+	    r=ParseToken(p, &buf);
+	    DBufPuts(&pre_buf, DBufValue(&buf));
+	    DBufPutc(&pre_buf, ' ');
+	    DBufFree(&buf);
+	    if (r) {
+		FreeTrig(&trig);
+		return r;
+	    }
+	    (void) sscanf(DBufValue(&pre_buf), "%d %d %d",
+			  &col_r, &col_g, &col_b);
+	    if (col_r < 0) col_r = 0;
+	    else if (col_r > 255) col_r = 255;
+	    if (col_g < 0) col_g = 0;
+	    else if (col_g > 255) col_g = 255;
+	    if (col_b < 0) col_b = 0;
+	    else if (col_b > 255) col_b = 255;
+	    if (!PsCal && !DoSimpleCalendar) {
+		DBufFree(&pre_buf);
+	    }
+	}
     }
 
     /* If trigger date == today, add it to the current entry */
@@ -859,12 +1077,14 @@ static int DoCalRem(ParsePtr p, int col)
 		if (DBufPuts(&obuf, SimpleTime(NO_TIME)) != OK) {
 		    DBufFree(&obuf);
 		    DBufFree(&pre_buf);
+		    FreeTrig(&trig);
 		    return E_NO_MEM;
 		}
 	    } else {
 		if (DBufPuts(&obuf, CalendarTime(tim.ttime, tim.duration)) != OK) {
 		    DBufFree(&obuf);
 		    DBufFree(&pre_buf);
+		    FreeTrig(&trig);
 		    return E_NO_MEM;
 		}
 	    }
@@ -881,6 +1101,7 @@ static int DoCalRem(ParsePtr p, int col)
 			DestroyValue(v);
 			DBufFree(&obuf);
 			DBufFree(&pre_buf);
+			FreeTrig(&trig);
 			return E_NO_MEM;
 		    }
 		}
@@ -899,11 +1120,13 @@ static int DoCalRem(ParsePtr p, int col)
 	if (r) {
 	    DBufFree(&pre_buf);
 	    DBufFree(&obuf);
+	    FreeTrig(&trig);
 	    return r;
 	}
 	if (DBufLen(&obuf) <= oldLen) {
 	    DBufFree(&obuf);
 	    DBufFree(&pre_buf);
+	    FreeTrig(&trig);
 	    return OK;
 	}
 	if (trig.typ != PASSTHRU_TYPE &&
@@ -918,6 +1141,7 @@ static int DoCalRem(ParsePtr p, int col)
 			DestroyValue(v);
 			DBufFree(&obuf);
 			DBufFree(&pre_buf);
+			FreeTrig(&trig);
 			return E_NO_MEM;
 		    }
 		}
@@ -929,30 +1153,37 @@ static int DoCalRem(ParsePtr p, int col)
 	DBufPuts(&pre_buf, s);
 	s = DBufValue(&pre_buf);
 	e = NEW(CalEntry);
+	if (!e) {
+	    DBufFree(&obuf);
+	    DBufFree(&pre_buf);
+	    FreeTrig(&trig);
+	    return E_NO_MEM;
+	}
+#ifdef REM_USE_WCHAR
+	e->wc_pos = NULL;
+	e->wc_text = NULL;
+#endif
 	e->is_color = is_color;
 	e->r = col_r;
 	e->g = col_g;
 	e->b = col_b;
-	if (!e) {
-	    DBufFree(&obuf);
-	    DBufFree(&pre_buf);
-	    return E_NO_MEM;
-	}
 	e->text = StrDup(s);
 	DBufFree(&obuf);
 	DBufFree(&pre_buf);
 	if (!e->text) {
 	    free(e);
+	    FreeTrig(&trig);
 	    return E_NO_MEM;
 	}
-	StrnCpy(e->tag, trig.tag, TAG_LEN);
-	if (!e->tag[0]) {
-	    if (SynthesizeTags) {
-		SynthesizeTag(e->tag);
-	    } else {
-		strcpy(e->tag, "*");
-	    }
+	make_wchar_versions(e);
+	DBufInit(&(e->tags));
+	DBufPuts(&(e->tags), DBufValue(&(trig.tags)));
+	if (SynthesizeTags) {
+	    AppendTag(&(e->tags), SynthesizeTag());
 	}
+
+	/* Don't need tags any more */
+	FreeTrig(&trig);
 	e->duration = tim.duration;
 	e->priority = trig.priority;
 	e->filename = StrDup(FileName);
@@ -1002,7 +1233,11 @@ static void WriteSimpleEntries(int col, int jul)
 	} else {
 	    printf(" *");
 	}
-	printf(" %s ", e->tag);
+	if (*DBufValue(&(e->tags))) {
+	    printf(" %s ", DBufValue(&(e->tags)));
+	} else {
+	    printf(" * ");
+	}
 	if (e->duration != NO_TIME) {
 	    printf("%d ", e->duration);
 	} else {
@@ -1014,8 +1249,11 @@ static void WriteSimpleEntries(int col, int jul)
 	    printf("* ");
 	}
 	printf("%s\n", e->text);
-	free((char *) e->text);
-	free((char *) e->filename);
+	free((void *)e->text);
+	free((void *)e->filename);
+#ifdef REM_USE_WCHAR
+	if (e->wc_text) free((void *)e->wc_text);
+#endif
 	n = e->next;
 	free(e);
 	e = n;
@@ -1115,9 +1353,9 @@ static void WriteCalDays(void)
     goff();
     for (i=0; i<7; i++) {
 	if (!MondayFirst)
-	    PrintCentered(DayName[(i+6)%7], ColSpaces, ' ');
+	    PrintCentered(DayName[(i+6)%7], ColSpaces, " ");
 	else
-	    PrintCentered(DayName[i%7], ColSpaces, ' ');
+	    PrintCentered(DayName[i%7], ColSpaces, " ");
 	gon();
 	DRAW(tb);
 	goff();
@@ -1285,10 +1523,11 @@ static void SortCol(CalEntry **col)
     }
 }
 
-void SynthesizeTag(char *out)
+char const *SynthesizeTag(void)
 {
     struct MD5Context ctx;
     unsigned char buf[16];
+    static char out[128];
     MD5Init(&ctx);
     MD5Update(&ctx, (unsigned char *) CurLine, strlen(CurLine));
     MD5Final(buf, &ctx);
@@ -1301,5 +1540,6 @@ void SynthesizeTag(char *out)
 	    (unsigned int) buf[10], (unsigned int) buf[11],
 	    (unsigned int) buf[12], (unsigned int) buf[13],
 	    (unsigned int) buf[14], (unsigned int) buf[15]);
+    return out;
 }
 
