@@ -54,11 +54,12 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
     int d, m, y;
     int tim = tt->ttime;
     int h, min, hh, ch, cmin, chh;
-    char *pm, *cpm;
+    char const *pm, *cpm;
     int tdiff, adiff, mdiff, hdiff;
-    char *mplu, *hplu, *when, *plu;
+    char const *mplu, *hplu, *when, *plu;
     int has_quote = 0;
-    char *ss, *os;
+    char *ss;
+    char *os;
     char s[256];
     int origLen = DBufLen(dbuf);
 
@@ -649,7 +650,7 @@ int DoSubst(ParsePtr p, DynamicBuffer *dbuf, Trigger *t, TimeTrig *tt, int jul,
 /*  default triggers and a mode of NORMAL_MODE.                */
 /*                                                             */
 /***************************************************************/
-int DoSubstFromString(char *source, DynamicBuffer *dbuf,
+int DoSubstFromString(char const *source, DynamicBuffer *dbuf,
 			     int jul, int tim)
 {
     Trigger tempTrig;
