@@ -1335,10 +1335,11 @@ static int FShell(func_info *info)
 /***************************************************************/
 static int FIsomitted(func_info *info)
 {
+    int r;
     if (!HASDATE(ARG(0))) return E_BAD_TYPE;
 
     RetVal.type = INT_TYPE;
-    int r = IsOmitted(DATEPART(ARG(0)), 0, NULL, &RETVAL);
+    r = IsOmitted(DATEPART(ARG(0)), 0, NULL, &RETVAL);
     return r;
 }
 
@@ -2659,7 +2660,10 @@ FEvalTrig(func_info *info)
     p.allownested = 0;
     r = ParseRem(&p, &trig, &tim, 0);
     if (r) return r;
-    if (trig.typ != NO_TYPE) return E_PARSE_ERR;
+    if (trig.typ != NO_TYPE) {
+	FreeTrig(&trig);
+	return E_PARSE_ERR;
+    }
     if (scanfrom == NO_DATE) {
 	jul = ComputeTrigger(trig.scanfrom, &trig, &r, 0);
     } else {
@@ -2669,6 +2673,7 @@ FEvalTrig(func_info *info)
 	}
 	jul = ComputeTrigger(scanfrom, &trig, &r, 0);
     }
+    FreeTrig(&trig);
     if (r) return r;
     if (jul < 0) {
 	RetVal.type = INT_TYPE;
