@@ -352,6 +352,11 @@ void InitRemind(int argc, char const *argv[])
 			arg++;
 			continue;
 		    }
+		    if (*arg == 'u' || *arg == 'U') {
+			UseUTF8Chars = 1;
+			arg++;
+			continue;
+		    }
 		    if (*arg == 'c' || *arg == 'C') {
 		        UseVTColors = 1;
 		        arg++;
@@ -785,6 +790,7 @@ static void InitializeVar(char const *str)
     return;
 }
 
+#if defined(__APPLE__) || defined(__CYGWIN__)
 static char const pmsg1[] = {
     0x4c, 0x62, 0x68, 0x20, 0x6e, 0x63, 0x63, 0x72, 0x6e, 0x65, 0x20,
     0x67, 0x62, 0x20, 0x6f, 0x72, 0x20, 0x65, 0x68, 0x61, 0x61, 0x76,
@@ -815,7 +821,6 @@ static char const pmsg2[] = {
     0x67, 0x6e, 0x65, 0x76, 0x79, 0x6c, 0x2e, 0x0a, 0x00
 };
 
-#if defined(__APPLE__) || defined(__CYGWIN__)
 static void
 rkrphgvba(int x)
 {
