@@ -280,7 +280,7 @@ EXTERN char *ErrMsg[] =
     "Peruutusarvo annettu kahdesti",
     "ONCE-avainsanaa k\xE4ytetty kahdesti. (Hah.)",
     "AT-sanan per\xE4st\xE4 puuttuu aika",
-    "UNTIL-sanaa k\xE4ytetty kahdesti",
+    "THROUGH/UNTIL-sanaa k\xE4ytetty kahdesti",
     "Ep\xE4t\xE4ydellinen p\xE4iv\xE4ys",
     "FROM/SCANFROM-sanaa k\xE4ytetty kahdesti",
     "Muuttuja",
@@ -384,7 +384,7 @@ EXTERN char *ErrMsg[] =
     "Peruutusarvo annettu kahdesti",
     "ONCE-avainsanaa k\x84ytetty kahdesti. (Hah.)",
     "AT-sanan per\x84st\x84 puuttuu aika",
-    "UNTIL-sanaa k\x84ytetty kahdesti",
+    "THROUGH/UNTIL-sanaa k\x84ytetty kahdesti",
     "Ep\x84t\x84ydellinen p\x84iv\x84ys",
     "FROM/SCANFROM-sanaa k\x84ytetty kahdesti",
     "Muuttuja",
@@ -488,7 +488,7 @@ EXTERN char *ErrMsg[] =
     "Peruutusarvo annettu kahdesti",
     "ONCE-avainsanaa k{ytetty kahdesti. (Hah.)",
     "AT-sanan per{st{ puuttuu aika",
-    "UNTIL-sanaa k{ytetty kahdesti",
+    "THROUGH/UNTIL-sanaa k{ytetty kahdesti",
     "Ep{t{ydellinen p{iv{ys",
     "FROM/SCANFROM-sanaa k{ytetty kahdesti",
     "Muuttuja",
