@@ -252,7 +252,7 @@ EXTERN char *ErrMsg[] =
     "Warto\266\346 cofni\352cia podana dw\363krotnie",
     "S\263owo ONCE u\277yte dw\363krotnie.",
     "Po AT oczekiwany jest czas",
-    "S\263owo UNTIL u\277yte dw\363krotnie",
+    "S\263owo THROUGH/UNTIL u\277yte dw\363krotnie",
     "Niekompletna specyfikacja daty",
     "S\263owo FROM/SCANFROM u\277yte dw\363krotnie",
     "Zmienna",
@@ -355,7 +355,7 @@ EXTERN char *ErrMsg[] =
     "Wartosc cofniecia podana dwokrotnie",
     "Slowo ONCE uzyte dwokrotnie.",
     "Po AT oczekiwany jest czas",
-    "Slowo UNTIL uzyte dwokrotnie",
+    "Slowo THROUGH/UNTIL uzyte dwokrotnie",
     "Niekompletna specyfikacja daty",
     "Slowo FROM/SCANFROM uzyte dwokrotnie",
     "Zmienna",
