@@ -19,6 +19,9 @@
 #include <signal.h>
 #include <string.h>
 #include <stdarg.h>
+#ifdef HAVE_LOCALE_H
+#include <locale.h>
+#endif
 
 #include <ctype.h>
 #ifdef TIME_WITH_SYS_TIME
@@ -57,6 +60,10 @@ int main(int argc, char *argv[])
 {
     int pid;
 
+#ifdef HAVE_SETLOCALE
+    setlocale(LC_ALL, "");
+#endif
+
 /* The very first thing to do is to set up ErrFp to be stderr */
     ErrFp = stderr;
 
@@ -806,6 +813,7 @@ int DoIfTrig(ParsePtr p)
 		}
 	    }
 	}
+	FreeTrig(&trig);
     }
     NumIfs++;
     IfFlags &= ~(IF_MASK << (2*NumIfs - 2));
@@ -1299,3 +1307,18 @@ void SigIntHandler(int d)
     GotSigInt();
     exit(0);
 }
+
+void
+AppendTag(DynamicBuffer *buf, char const *s)
+{
+    if (*(DBufValue(buf))) {
+	DBufPutc(buf, ',');
+    }
+    DBufPuts(buf, s);
+}
+
+void
+FreeTrig(Trigger *t)
+{
+    DBufFree(&(t->tags));
+}
