@@ -125,7 +125,7 @@ int PushOmitContext(ParsePtr p)
 	free(context);
 	return E_NO_MEM;
     }
-      
+
 /* Copy the context over */
     for (i=0; i<NumFullOmits; i++)
 	*(context->fullsave + i) = FullOmitArray[i];
@@ -272,6 +272,9 @@ static void InsertIntoSortedArray(int *array, int num, int key)
     *cur = key;
 }
 
+static int DoThroughOmit(ParsePtr p, int y, int m, int d);
+static void DumpOmits(void);
+
 /***************************************************************/
 /*                                                             */
 /*  DoOmit                                                     */
@@ -285,15 +288,25 @@ int DoOmit(ParsePtr p)
     Token tok;
     int parsing=1;
     int syndrome;
+    int not_first_token = -1;
 
     DynamicBuffer buf;
     DBufInit(&buf);
 
 /* Parse the OMIT.  We need a month and day; year is optional. */
     while(parsing) {
+	not_first_token++;
 	if ( (r=ParseToken(p, &buf)) ) return r;
 	FindToken(DBufValue(&buf), &tok);
 	switch (tok.type) {
+	case T_Dumpvars:
+	    if (not_first_token) return E_PARSE_ERR;
+	    DBufFree(&buf);
+	    r = VerifyEoln(p);
+	    if (r != OK) return r;
+	    DumpOmits();
+	    return OK;
+
 	case T_Date:
 	    DBufFree(&buf);
 	    if (y != NO_YR) return E_YR_TWICE;
@@ -324,6 +337,11 @@ int DoOmit(ParsePtr p)
 	    DBufFree(&buf);
 	    break;
 
+	case T_Through:
+	    DBufFree(&buf);
+	    if (y == NO_YR || m == NO_MON || d == NO_DAY) return E_INCOMPLETE;
+	    return DoThroughOmit(p, y, m, d);
+
 	case T_Empty:
 	case T_Comment:
 	case T_RemType:
@@ -362,7 +380,123 @@ int DoOmit(ParsePtr p)
 	    NumFullOmits++;
 	}
     }
-    if (tok.type == T_RemType || tok.type == T_Priority) return E_PARSE_AS_REM;
+    if (tok.type == T_Tag || tok.type == T_Duration || tok.type == T_RemType || tok.type == T_Priority) return E_PARSE_AS_REM;
     return OK;
 
 }
+
+static int
+DoThroughOmit(ParsePtr p, int ystart, int mstart, int dstart)
+{
+    int yend = NO_YR, mend = NO_MON, dend = NO_DAY, r;
+    int start, end, tmp;
+
+    Token tok;
+
+    DynamicBuffer buf;
+    DBufInit(&buf);
+    int parsing = 1;
+
+    while(parsing) {
+	if ( (r=ParseToken(p, &buf)) ) return r;
+	FindToken(DBufValue(&buf), &tok);
+
+	switch(tok.type) {
+	case T_Date:
+	    DBufFree(&buf);
+	    if (yend != NO_YR) return E_YR_TWICE;
+	    if (mend != NO_MON) return E_MON_TWICE;
+	    if (dend != NO_DAY) return E_DAY_TWICE;
+	    FromJulian(tok.val, &yend, &mend, &dend);
+	    break;
+
+	case T_Year:
+	    DBufFree(&buf);
+	    if (yend != NO_YR) return E_YR_TWICE;
+	    yend = tok.val;
+	    break;
+
+	case T_Month:
+	    DBufFree(&buf);
+	    if (mend != NO_MON) return E_MON_TWICE;
+	    mend = tok.val;
+	    break;
+
+	case T_Day:
+	    DBufFree(&buf);
+	    if (dend != NO_DAY) return E_DAY_TWICE;
+	    dend = tok.val;
+	    break;
+
+	case T_Empty:
+	case T_Comment:
+	case T_RemType:
+	case T_Priority:
+	case T_Tag:
+	case T_Duration:
+	    DBufFree(&buf);
+	    parsing = 0;
+	    break;
+
+	default:
+	    Eprint("%s: `%s' (OMIT)", ErrMsg[E_UNKNOWN_TOKEN],
+		   DBufValue(&buf));
+	    DBufFree(&buf);
+	    return E_UNKNOWN_TOKEN;
+
+	}
+    }
+    if (yend == NO_YR || mend == NO_MON || dend == NO_DAY) return E_INCOMPLETE;
+    if (dend > DaysInMonth(mend, yend)) return E_BAD_DATE;
+    if (dstart > DaysInMonth(mstart, ystart)) return E_BAD_DATE;
+
+    start = Julian(ystart, mstart, dstart);
+    end   = Julian(yend,   mend,   dend);
+
+    if (end < start) {
+	tmp = start;
+	start = end;
+	end = tmp;
+    }
+
+    tmp = end - start + 1;
+
+    /* Don't create any OMITs if there would be too many. */
+    if (NumFullOmits + tmp >= MAX_FULL_OMITS) return E_2MANY_FULL;
+    for (tmp = start; tmp <= end; tmp++) {
+	if (!BexistsIntArray(FullOmitArray, NumFullOmits, tmp)) {
+	    InsertIntoSortedArray(FullOmitArray, NumFullOmits, tmp);
+	    NumFullOmits++;
+	}
+    }
+    if (tok.type == T_Tag || tok.type == T_Duration || tok.type == T_RemType || tok.type == T_Priority) return E_PARSE_AS_REM;
+    return OK;
+}
+
+void
+DumpOmits(void)
+{
+    int i;
+    int y, m, d;
+    printf("Global Full OMITs (%d of maximum allowed %d):\n", NumFullOmits, MAX_FULL_OMITS);
+    if (!NumFullOmits) {
+	printf("\tNone.\n");
+    } else {
+	for (i=0; i<NumFullOmits; i++) {
+	    FromJulian(FullOmitArray[i], &y, &m, &d);
+	    printf("\t%04d%c%02d%c%02d\n",
+		    y, DateSep, m+1, DateSep, d);
+	}
+    }
+    printf("Global Partial OMITs (%d of maximum allowed %d):\n", NumPartialOmits, MAX_PARTIAL_OMITS);
+    if (!NumPartialOmits) {
+	printf("\tNone.\n");
+    } else {
+	for (i=0; i<NumPartialOmits; i++) {
+	    m = PartialOmitArray[i] >> 5 & 0xf;
+	    d = PartialOmitArray[i] & 0x1f;
+	    printf("\t%02d%c%02d\n", m+1, DateSep, d);
+	}
+    }
+}
+
