@@ -28,8 +28,20 @@
 #define SPECIAL_POSTSCRIPT 1
 #define SPECIAL_PSFILE     2
 #define SPECIAL_MOON       3
-#define SPECIAL_SHADE      4
-#define SPECIAL_COLOR      5
+#define SPECIAL_COLOR      4
+#define SPECIAL_WEEK       5
+#define SPECIAL_SHADE      6
+
+/* Array holding how specials sort */
+static int SpecialSortOrder[] = {
+    0, /* NORMAL */
+    1, /* POSTSCRIPT */
+    1, /* PSFILE */
+    2, /* MOON */
+    0, /* COLOR */
+    4, /* WEEK */
+    5  /* SHADE */
+};
 
 typedef struct calentry {
     struct calentry *next;
@@ -185,7 +197,7 @@ void DoPsCal(void)
     char const *startOfBody;
     char passthru[PASSTHRU_LEN+1];
     DynamicBuffer buf;
-    CalEntry *c, *d;
+    CalEntry *c, *d, *p;
 
 /* Read the month and year name, followed by # days in month and 1st day of
    month */
@@ -306,6 +318,7 @@ void DoPsCal(void)
 	if (!strcmp(passthru, "PostScript") ||
 	    !strcmp(passthru, "PSFile") ||
 	    !strcmp(passthru, "MOON") ||
+	    !strcmp(passthru, "WEEK") ||
 	    !strcmp(passthru, "SHADE")) {
 	    is_ps = 1;
 	}
@@ -324,6 +337,8 @@ void DoPsCal(void)
 		c->special = SPECIAL_SHADE;
 	    } else if (!strcmp(passthru, "MOON")) {
 		c->special = SPECIAL_MOON;
+	    } else if (!strcmp(passthru, "WEEK")) {
+		c->special = SPECIAL_WEEK;
 	    } else {
 		c->special = SPECIAL_PSFILE;
 	    }
@@ -332,8 +347,23 @@ void DoPsCal(void)
 		PsEntries[DayNum] = c;
 	    } else {
 		d = PsEntries[DayNum];
-		while(d->next) d = d->next;
-		d->next = c;
+		p = NULL;
+		/* Slot it into the right place */
+		while (d->next && (SpecialSortOrder[c->special] <= SpecialSortOrder[d->special])) {
+		    p = d;
+		    d = d->next;
+		}
+		if (SpecialSortOrder[c->special] <= SpecialSortOrder[d->special]) {
+		    c->next = d->next;
+		    d->next = c;
+		} else {
+		    if (p) {
+			p->next = c;
+		    } else {
+			PsEntries[DayNum] = c;
+		    }
+		    c->next = d;
+		}
 	    }
 	} else if (!strcmp(passthru, "*") ||
 	           !strcmp(passthru, "COLOR")) {
@@ -887,6 +917,7 @@ int DoQueuedPs(void)
     int fnoff;
     char buffer[512];
     char const *size, *extra;
+    char const *s;
     int num, r, g, b, phase, fontsize, moonsize;
     unsigned char c;
 
@@ -954,6 +985,22 @@ int DoQueuedPs(void)
 		       r/255.0, g/255.0, b/255.0);
 		break;
 
+	    case SPECIAL_WEEK:          /* Week number */
+		printf("gsave Border Border 2 div moveto /EntryFont findfont EntrySize 1.2 div scalefont setfont (");
+		s = e->entry+fnoff;
+		while(*s && isspace(*s)) {
+		    s++;
+		}
+		while(*s) {
+		    if (*s == '\\' || *s == '(' || *s == ')') {
+			PutChar('\\');
+		    }
+		    PutChar(*s);
+		    s++;
+		}
+		printf(") show grestore\n");
+		break;
+
 	    case SPECIAL_MOON:		/* Moon phase */
 		num = sscanf(e->entry+fnoff, "%d %d %d", &phase, &moonsize,
 			     &fontsize);
