@@ -206,16 +206,27 @@ static int NextSimpleTrig(int startdate, Trigger *trig, int *err)
 	    *err = E_BAD_DATE;
 	    return -1;
 	}
+	/* Back up a year in case we'll cross a year boundary*/
+	if (y > BASE) {
+	    y--;
+	}
+
 	/* Move up to the first valid year */
 	while (trig->d > DaysInMonth(trig->m, y)) y++;
 
+	/* Try last year */
+	j = Julian(y, trig->m, trig->d);
+	while(! (trig->wd & (1 << (j%7)))) j++;
+	if (j >= startdate) return j;
+
 	/* Try this year */
+	y++;
 	j = Julian(y, trig->m, trig->d);
 	while(! (trig->wd & (1 << (j%7)))) j++;
 	if (j >= startdate) return j;
 
 	/* Must be next year */
-	y = y + 1;
+	y++;
 	while (trig->d > DaysInMonth(trig->m, y)) y++;
 	j = Julian(y, trig->m, trig->d);
 	while(! (trig->wd & (1 << (j%7)))) j++;
@@ -286,16 +297,33 @@ static int JYear(int jul)
 /***************************************************************/
 static int GetNextTriggerDate(Trigger *trig, int start, int *err, int *nextstart)
 {
-    int simple, mod;
+    int simple, mod, omit;
 
 /* First:  Have we passed the UNTIL date? */
     if (trig->until != NO_UNTIL &&
-	trig->until < start) return -1; /* expired */
+	trig->until < start) {
+	trig->expired = 1;
+	return -1; /* expired */
+    }
 
 /* Next: If it's an "AFTER"-type skip, back up
    until we're at the start of a block of holidays */
-    if (trig->skip == AFTER_SKIP)
-	while (IsOmitted(start-1, trig->localomit)) start--;
+    if (trig->skip == AFTER_SKIP) {
+	int iter = 0;
+	while (iter++ <= MaxSatIter) {
+	    *err = IsOmitted(start-1, trig->localomit, trig->omitfunc, &omit);
+	    if (*err) return -2;
+	    if (!omit) {
+		break;
+	    }
+	    start--;
+	}
+	if (iter > MaxSatIter) {
+	    /* omitfunc must have returned "true" too often */
+	    *err = E_CANT_TRIG;
+	    return -2;
+	}
+    }
 
 /* Find the next simple trigger */
     simple = NextSimpleTrig(start, trig, err);
@@ -309,12 +337,29 @@ static int GetNextTriggerDate(Trigger *trig, int start, int *err, int *nextstart
 /* If there's a BACK, back up... */
     if (trig->back != NO_BACK) {
 	mod = trig->back;
-	if (mod < 0) simple += mod;
-	else
-	    while(mod) {
-		simple--;
-		if (!IsOmitted(simple, trig->localomit)) mod--;
+	if (mod < 0) {
+	    simple += mod;
+	}
+	else {
+	    int iter = 0;
+	    int max = MaxSatIter;
+	    if (max < mod*2) {
+		max = mod*2;
 	    }
+	    while(iter++ <= max) {
+		if (!mod) {
+		    break;
+		}
+		simple--;
+		*err = IsOmitted(simple, trig->localomit, trig->omitfunc, &omit);
+		if (*err) return -2;
+		if (!omit) mod--;
+	    }
+	    if (iter > max) {
+	        *err = E_CANT_TRIG;
+		return -2;
+	    }
+	}
     }
 
 /* If there's a REP, calculate the next occurrence */
@@ -327,12 +372,38 @@ static int GetNextTriggerDate(Trigger *trig, int start, int *err, int *nextstart
     }
 
 /* If it's a "BEFORE"-type skip, back up */
-    if (trig->skip == BEFORE_SKIP)
-	while(IsOmitted(simple, trig->localomit)) simple--;
+    if (trig->skip == BEFORE_SKIP) {
+	int iter = 0;
+	while(iter++ <= MaxSatIter) {
+	    *err = IsOmitted(simple, trig->localomit, trig->omitfunc, &omit);
+	    if (*err) return -2;
+	    if (!omit) {
+		break;
+	    }
+	    simple--;
+	}
+	if (iter > MaxSatIter) {
+	    *err = E_CANT_TRIG;
+	    return -2;
+	}
+    }
 
 /* If it's an "AFTER"-type skip, jump ahead */
-    if (trig->skip == AFTER_SKIP)
-	while (IsOmitted(simple, trig->localomit)) simple++;
+    if (trig->skip == AFTER_SKIP) {
+	int iter = 0;
+	while (iter++ <= MaxSatIter) {
+	    *err = IsOmitted(simple, trig->localomit, trig->omitfunc, &omit);
+	    if (*err) return -2;
+	    if (!omit) {
+		break;
+	    }
+	    simple++;
+	}
+	if (iter > MaxSatIter) {
+	    *err = E_CANT_TRIG;
+	    return -2;
+	}
+    }
 
 /* Return the date */
     return simple;
@@ -346,15 +417,16 @@ static int GetNextTriggerDate(Trigger *trig, int start, int *err, int *nextstart
 /*  today's date.                                              */
 /*                                                             */
 /***************************************************************/
-int ComputeTrigger(int today, Trigger *trig, int *err)
+int ComputeTrigger(int today, Trigger *trig, int *err, int save_in_globals)
 {
     int nattempts = 0,
 	start = today,
 	nextstart = 0,
-	y, m, d,
+	y, m, d, omit,
 	result;
 
-    LastTrigValid = 0;
+    trig->expired = 0;
+    if (save_in_globals) LastTrigValid = 0;
 
 /* Assume everything works */
     *err = OK;
@@ -373,14 +445,15 @@ int ComputeTrigger(int today, Trigger *trig, int *err)
 	*err = E_REP_FULSPEC;
 	return -1;
     }
-       
-   
+
+
     while (nattempts++ < TRIG_ATTEMPTS) {
 	result = GetNextTriggerDate(trig, start, err, &nextstart);
 
 	/* If there's an error, die immediately */
 	if (*err) return -1;
 	if (result == -1) {
+	    trig->expired = 1;
 	    if (DebugFlag & DB_PRTTRIG) {
 		fprintf(ErrFp, "%s(%d): %s\n",
 			FileName, LineNo, ErrMsg[E_EXPIRED]);
@@ -389,10 +462,18 @@ int ComputeTrigger(int today, Trigger *trig, int *err)
 	}
 
 	/* If result is >= today, great! */
+	if (trig->skip == SKIP_SKIP) {
+	    *err = IsOmitted(result, trig->localomit, trig->omitfunc, &omit);
+	    if (*err) return -1;
+	} else {
+	    omit = 0;
+	}
 	if (result >= today &&
-	    (trig->skip != SKIP_SKIP || !IsOmitted(result, trig->localomit))) {
-	    LastTriggerDate = result;  /* Save in global var */
-	    LastTrigValid = 1;
+	    (trig->skip != SKIP_SKIP || !omit)) {
+	    if (save_in_globals) {
+		LastTriggerDate = result;  /* Save in global var */
+		LastTrigValid = 1;
+	    }
 	    if (DebugFlag & DB_PRTTRIG) {
 		FromJulian(result, &y, &m, &d);
 		fprintf(ErrFp, "%s(%d): Trig = %s, %d %s, %d\n",
@@ -409,19 +490,22 @@ int ComputeTrigger(int today, Trigger *trig, int *err)
 	if (trig->back == NO_BACK &&
 	    trig->skip == NO_SKIP &&
 	    trig->rep == NO_REP) {
+	    trig->expired = 1;
 	    if (DebugFlag & DB_PRTTRIG) {
 		fprintf(ErrFp, "%s(%d): %s\n",
 			FileName, LineNo, ErrMsg[E_EXPIRED]);
 	    }
 	    if (result != -1) {
-	        LastTriggerDate = result;
-		LastTrigValid = 1;
+		if (save_in_globals) {
+		    LastTriggerDate = result;
+		    LastTrigValid = 1;
+		}
 	    }
 	    return -1;
 	}
 
 	if (trig->skip == SKIP_SKIP &&
-	    IsOmitted(result, trig->localomit) &&
+	    omit &&
 	    nextstart <= start &&
 	    result >= start) {
 	    nextstart = result + 1;
@@ -430,9 +514,12 @@ int ComputeTrigger(int today, Trigger *trig, int *err)
 	/* Keep scanning... unless there's no point in doing it.*/
 	if (nextstart <= start) {
 	    if (result != -1) {
-		LastTriggerDate = result;
-		LastTrigValid = 1;
+		if (save_in_globals) {
+		    LastTriggerDate = result;
+		    LastTrigValid = 1;
+		}
 	    }
+	    trig->expired = 1;
 	    if (DebugFlag & DB_PRTTRIG) {
 		fprintf(ErrFp, "%s(%d): %s\n",
 			FileName, LineNo, ErrMsg[E_EXPIRED]);
