@@ -40,7 +40,7 @@ static UserFunc *FuncHash[FUNC_HASH_SIZE];
 
 /* Access to built-in functions */
 extern int NumFuncs;
-extern Operator Func[];
+extern BuiltinFunc Func[];
 
 /* We need access to the expression evaluation stack */
 extern Value ValStack[];
@@ -243,7 +243,7 @@ static void FSet(UserFunc *f)
 /*  Call a user-defined function.                              */
 /*                                                             */
 /***************************************************************/
-int CallUserFunc(char const *name, int nargs)
+int CallUserFunc(char const *name, int nargs, ParsePtr p)
 {
     UserFunc *f;
     int h = HashVal(name) % FUNC_HASH_SIZE;
@@ -299,9 +299,9 @@ int CallUserFunc(char const *name, int nargs)
     s = f->text;
 
     /* Skip the opening bracket, if there's one */
-    while (isspace(*s)) s++;
+    while (isempty(*s)) s++;
     if (*s == BEG_OF_EXPR) s++;
-    h = Evaluate(&s, f->locals);
+    h = Evaluate(&s, f->locals, p);
     f->IsActive = 0;
     DestroyLocalVals(f);
     if (DebugFlag &DB_PRTEXPR) {
